#include "utils.h"
#include "constants.h"
#include "resourcemanager.h"

#include <alleggl.h>
#include <GL/glu.h>
#include <stdexcept>
#include <cmath>

void SetOrthoProjection()
{
	glDisable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
  	glLoadIdentity();
	glOrtho(0, ScreenWidth, 0, ScreenHeight, -1, 1);
	glMatrixMode(GL_MODELVIEW);
}

void CheckGlError()
{
	GLenum err = glGetError();
	if (err != GL_NO_ERROR) {
		throw std::runtime_error("OpenGL error :" + err);
	}
}


/*
 * The following function was lifted from:
 *
 * http://astronomy.swin.edu.au/~pbourke/opengl/sphere/sphere.c
 *
 * Credited to Federico Dosil
 */
 
/*
   Create a sphere centered at c, with radius r, and precision n
   Draw a point for zero radius spheres
   Use CCW facet ordering
   "method" is 0 for quads, 1 for triangles
      (quads look nicer in wireframe mode)
   Partial spheres can be created using theta1->theta2, phi1->phi2
   in radians 0 < theta < 2pi, -pi/2 < phi < pi/2
*/
void DrawSphere(Vect3d c,double r,int n,int method,
	double theta1,double theta2,double phi1,double phi2)
{
	int i,j;
	double jdivn,j1divn,idivn,dosdivn,unodivn=1/(double)n,ndiv2=(double)n/2,t1,t2,t3,cost1,cost2,cte1,cte3;
	cte3 = (theta2-theta1)/n;
	cte1 = (phi2-phi1)/ndiv2;
	dosdivn = 2*unodivn;
	Vect3d e,p,e2,p2;
	/* Handle special cases */
	if (r < 0)
		r = -r;
	if (n < 0){
		n = -n;
		ndiv2 = -ndiv2;
	}
	/* No need to do this.
	if (n < 4 || r <= 0) {
		glBegin(GL_POINTS);
		glVertex3f(c.x,c.y,c.z);
		glEnd();
		return;
	}
	*/
	
	t2=phi1;
	cost2=cos(phi1);
	j1divn=0;
	for (j=0;j<ndiv2;j++) {
		t1 = t2;//t1 = phi1 + j * cte1;
		t2 += cte1;//t2 = phi1 + (j + 1) * cte1;
		t3 = theta1 - cte3;
		cost1 = cost2;//cost1=cos(t1);
		cost2 = cos(t2);
		e.y = sin(t1);
		e2.y = sin(t2);
		p.y = c.y + r * e.y;
		p2.y = c.y + r * e2.y;
	
		if (method == 0)
			glBegin(GL_QUAD_STRIP);
		else
			glBegin(GL_TRIANGLE_STRIP);
	
		idivn=0;
		jdivn=j1divn;
		j1divn+=dosdivn;//=2*(j+1)/(double)n;
		for (i=0;i<=n;i++) {
		//t3 = theta1 + i * (theta2 - theta1) / n;
			t3 += cte3;
			e.x = cost1 * cos(t3);
		//e.y = sin(t1);
			e.z = cost1 * sin(t3);
			p.x = c.x + r * e.x;
		//p.y = c.y + r * e.y;
			p.z = c.z + r * e.z;
			glNormal3f(e.x,e.y,e.z);
			glTexCoord2f(idivn,jdivn);
			glVertex3f(p.x,p.y,p.z);
	
			e2.x = cost2 * cos(t3);
		//e.y = sin(t2);
			e2.z = cost2 * sin(t3);
			p2.x = c.x + r * e2.x;
		//p.y = c.y + r * e.y;
			p2.z = c.z + r * e2.z;
			glNormal3f(e2.x,e2.y,e2.z);
			glTexCoord2f(idivn,j1divn);
			glVertex3f(p2.x,p2.y,p2.z);
			idivn += unodivn;
		}
		glEnd();
	}
}

float FloatRand()
{
	float r = (float) rand();
	return (r / (float) RAND_MAX);
}

void DrawCircle(Vect3d centre ,double radius) 
{
	const int numSteps = 20;
	double step = 360.0 / numSteps;
	
	glBegin(GL_LINE_LOOP);
	for (int n=0; n < numSteps; n++) {
		double theta = ((double) n ) * step;
		Vect3d where = centre + Vect3d::fromPolar(theta, radius);
		glVertex2f(where.x, where.y);
	}
	glEnd();
}

static void ClearIntermission()
{
	glClearColor(0,0,0.4, 0);
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	glDisable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
  	glLoadIdentity();
	glOrtho(0, ScreenWidth, 0, ScreenHeight, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glDisable(GL_BLEND);
}

void ShowIntermission(const std::string &caption)
{
	BITMAP *tempBitmap = create_bitmap(512, 16);
	clear_to_color(tempBitmap,0);
	int white = makecol(255,255,255);
	int black = 0;
	textout_ex(tempBitmap, resourceManager.getFont(), 
		caption.c_str(),1,1, white, black);
	GLuint tex = allegro_gl_make_texture_ex(
		AGL_TEXTURE_FLIP,
		tempBitmap,
		GL_LUMINANCE8);
	glBindTexture(GL_TEXTURE_2D, tex);
	destroy_bitmap(tempBitmap);
	
	// Intermission loop
	int counter = 200;
	while (counter > 0) {
		ClearIntermission();
		glEnable(GL_BLEND);
		glDisable(GL_LIGHTING);
		glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
		glEnable(GL_TEXTURE_2D);
		glColor3f(1,1,1);
		float yoffset = 350;
		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(0,yoffset);
		glTexCoord2f(1,0);
		glVertex2f(512,yoffset);
		glTexCoord2f(1,1);
		glVertex2f(512,32 + yoffset);
		glTexCoord2f(0,1);
		glVertex2f(0,32 + yoffset);
		glEnd();
		// draw quad.
		counter --;
		allegro_gl_flip();
		rest(20);
		if (key[KEY_SPACE] && (counter < 160)) {
			counter = 0;
		}
		if (key[KEY_ESC]) {
			counter = 0;
		}
	}
	
	glDeleteTextures(1, &tex);
}
